@@ -7,6 +7,7 @@ import logging
 import secrets
 import struct
 import sys
+import threading
 import uuid
 import plotly
 import dash
@@ -319,6 +320,12 @@ class DashBlueprint:
             layout = transform.layout(layout, self._layout_is_function)
         return layout
 
+    def embed(self, app: DashProxy):
+        if app.blueprint._layout_is_function and app._got_first_request["setup_server"]:
+            return self._layout_value()
+        self.register_callbacks(app)
+        return self._layout_value()
+
     @property
     def layout(self):
         return self._layout
@@ -345,6 +352,7 @@ class DashProxy(dash.Dash):
         super().__init__(*args, **kwargs)
         self.blueprint = DashBlueprint(transforms,
                                        include_global_callbacks=include_global_callbacks) if blueprint is None else blueprint
+        self.setup_server_lock = threading.Lock()
 
     def callback(self, *args, **kwargs):
         return self.blueprint.callback(*args, **kwargs)
@@ -364,18 +372,22 @@ class DashProxy(dash.Dash):
         self.blueprint.register_callbacks(super())
 
     def _setup_server(self):
-        first_request = not bool(self._got_first_request["setup_server"])
-        if first_request:
-            self.register_callbacks()
-        # Proceed as normally.
-        super()._setup_server()
-        if first_request:
-            # Remap callback bindings to enable callback registration via the 'before_first_request' hook.
-            self.callback = super().callback
-            self.clientside_callback = super().clientside_callback
-            # Set session secret. Used by some subclasses.
-            if not self.server.secret_key:
-                self.server.secret_key = secrets.token_urlsafe(16)
+        with self.setup_server_lock:
+            first_request = not bool(self._got_first_request["setup_server"])
+            if first_request:
+                # Trigger callback generation for embedded layouts.
+                if self.blueprint._layout_is_function:
+                    _ = self.blueprint.layout()
+                self.register_callbacks()
+            # Proceed as normally.
+            super()._setup_server()
+            if first_request:
+                # Remap callback bindings to enable callback registration via the 'before_first_request' hook.
+                self.callback = super().callback
+                self.clientside_callback = super().clientside_callback
+                # Set session secret. Used by some subclasses.
+                if not self.server.secret_key:
+                    self.server.secret_key = secrets.token_urlsafe(16)
 
     def hijack(self, app: dash.Dash):
         """
@@ -461,6 +473,14 @@ class DashTransform:
         return 1
 
 
+class StatefulDashTransform(DashTransform):
+
+    def __init__(self):
+        super().__init__()
+        self.blueprint = DashBlueprint()
+        self.components = []
+
+
 def _resolve_transforms(transforms: List[DashTransform]) -> List[DashTransform]:
     # Resolve transforms.
     transforms = [] if transforms is None else transforms
@@ -480,11 +500,9 @@ def _resolve_transforms(transforms: List[DashTransform]) -> List[DashTransform]:
 # region Blocking callback transform
 
 
-class BlockingCallbackTransform(DashTransform):
+class BlockingCallbackTransform(StatefulDashTransform):
     def __init__(self, timeout=60):
         super().__init__()
-        self.components = []
-        self.blueprint = DashBlueprint()
         self.timeout = timeout
 
     def transform_layout(self, layout):
@@ -756,11 +774,10 @@ def bind_logger(logger, single_output, out_flex_key):
 
 # region Cycle breaker transform
 
-class CycleBreakerTransform(DashTransform):
+class CycleBreakerTransform(StatefulDashTransform):
 
     def __init__(self):
         super().__init__()
-        self.components = []
 
     def transform_layout(self, layout):
         children = _as_list(layout.children) + self.components
@@ -1010,7 +1027,7 @@ def inject_proxies_recursively(node, proxy_map):
         node.children = children
 
 
-class MultiplexerTransform(DashTransform):
+class MultiplexerTransform(StatefulDashTransform):
     """
     The MultiplexerTransform makes it possible to target an output by callbacks multiple times. Under the hood, proxy
     components (dcc.Store) are used, and the proxy_location keyword argument determines where these proxies are placed.
@@ -1026,7 +1043,6 @@ class MultiplexerTransform(DashTransform):
         self.proxy_location = proxy_location
         self.proxy_map = defaultdict(lambda: [])
         self.proxy_wrapper_map = proxy_wrapper_map
-        self.blueprint = DashBlueprint()
 
     def transform_layout(self, layout):
         # Apply wrappers if needed.
@@ -1339,10 +1355,9 @@ class RedisStore(RedisCache):
 # region No output transform
 
 
-class NoOutputTransform(DashTransform):
+class NoOutputTransform(StatefulDashTransform):
     def __init__(self):
         super().__init__()
-        self.components = []
 
     def transform_layout(self, layout):
         children = _as_list(layout.children) + self.components
@@ -1468,12 +1483,10 @@ class DictOperator:
         return self._collect("clear")
 
 
-class OperatorTransform(DashTransform):
+class OperatorTransform(StatefulDashTransform):
     def __init__(self):
         super().__init__()
-        self.components = []
         self.operator_outputs = []
-        self.blueprint = DashBlueprint()
 
     def transform_layout(self, layout):
         children = _as_list(layout.children) + self.components
@@ -1555,6 +1568,7 @@ class OperatorTransform(DashTransform):
                             ...obj[idx],
                             ...x.obj
                         }};
+                        break;
                       // Unknown action(s).
                       default:
                         console.log("Received unknown action for component {original_id}.");
